--[[
Copy all mod-environment-l10n-texts into the game-root-l10n-text object.

Author: 	Ifko[nator]
Date: 		31.03.2019
Version:	1.0
]]

local root_i18n = getfenv(0).g_i18n;

for l10nTAG, text in pairs(g_i18n.texts) do
	root_i18n:setText(l10nTAG, text);
end;

--
-- animatedCharacter
-- specialisation for animatedCharacter
--
-- @author	112TEC 
-- @version 	v1.0
-- @date		11/04/2019
-- Modifikationen erst nach Rücksprache
-- Do not edit without my permission
-- 

local metadata = {
"## Title: animatedCharacter",
"## Notes: Animation einer Figur",
"## Author: 112TEC",
"## Version: 1.0",
"## Date: 19.04.2019",
"## Web: 112tec.schwaki.de"
}
animatedCharacter = {}
local animatedCharacter_mt = Class(animatedCharacter)

animatedCharacter.curModDir = g_currentModDirectory;

function animatedCharacter.onCreate(id)
	g_currentMission:addUpdateable(animatedCharacter:new(id))
end
function animatedCharacter:new(id)

	--print("--- loading animatedCharacter - specialisation for animated Character by 112TEC");

	local instance = {}
	setmetatable(instance, animatedCharacter_mt)
	instance.spline = getChildAt(id, 0)
	instance.animRoot = getChildAt(id, 1)
	setVisibility(instance.animRoot, false)

	instance.skeletonId = getChildAt(instance.animRoot, 0);
	instance.graphId = getChildAt(instance.animRoot, 1);
	instance.clipDistance = Utils.getNoNil(getUserAttribute(instance.animRoot, "playClipDistance"), 200);
	instance.startTime = Utils.getNoNil(getUserAttribute(instance.animRoot,"startTime"),0);
	instance.endTime = Utils.getNoNil(getUserAttribute(instance.animRoot,"endTime"),25);
	instance.animCharSet = getAnimCharacterSet(instance.skeletonId);
	if instance.animCharSet ~= 0 then
		instance.isEnabled = true;
		local clips = getUserAttribute(instance.animRoot, "clips");
		instance.clips = StringUtil.splitString(" ", clips);
		local walkClips = getUserAttribute(instance.animRoot, "walkClips");
		local speedScale = Utils.getNoNil(getUserAttribute(instance.animRoot,"speedScale"),1);
		instance.walkTracks = StringUtil.splitString(" ", walkClips);
		for i=1, #instance.walkTracks do
			instance.walkTracks[i] = tonumber(instance.walkTracks[i]);
		end;
		instance.numTracks = 0;
		for i=1, #instance.clips do
			local clip = getAnimClipIndex(instance.animCharSet, instance.clips[i]);
			if clip > -1 then
				assignAnimTrackClip(instance.animCharSet, i, clip);
				setAnimTrackLoopState(instance.animCharSet, i, true);
				setAnimTrackSpeedScale(instance.animCharSet, i, speedScale);
				instance.numTracks = instance.numTracks + 1;
			else
				print("animatedCharacter: clip "..instance.clips[i].." not found");
			end;
		end;
		instance.activeTrack = 1; 
		instance.nextTrack = 1;
		instance.timer1 = 0;
		instance.timer2 = 0;
		local soundFile = getUserAttribute(instance.animRoot, "soundFile");
		local soundradius = Utils.getNoNil(getUserAttribute(instance.animRoot,"soundRadius"),30);
		local soundinner = Utils.getNoNil(getUserAttribute(instance.animRoot,"soundInnerRadius"),15);
		local soundvolume = Utils.getNoNil(getUserAttribute(instance.animRoot,"soundVolume"),1);
		if soundFile then
			if string.find(soundFile, "data") == nil then
				soundFile = animatedCharacter.curModDir..soundFile;
			end;
			instance.sound = createAudioSource("AS", soundFile, soundradius, soundinner, soundvolume, 10000);
			local sample = getAudioSourceSample(instance.sound);
			instance.soundDuration = getSampleDuration(sample);
			link(instance.animRoot, instance.sound);
			setVisibility(instance.sound, false);
		end;
		instance.soundTimer = math.random(15000, 60000);
		instance.splinePosition = 0;
	end;
	setVisibility(instance.graphId, false);

	
	instance.viewable = false;
    instance.isActive = false;
    instance.oldIsActive = false;
  
    instance.timerId = addTimer(1000, "animatedCharacterTimerCallback", instance);



	local length = getSplineLength(instance.spline)
	if length ~= 0 then
		instance.splineLength = length
	end
	instance.initCount = 0


	return instance
end

function animatedCharacter:delete()
	removeTimer(self.timerId);
end;

function animatedCharacter:update(dt)

	if self.initCount > 0 then

	if self.isEnabled then
		if self.viewable then
			local cam = getCamera();
			local cx, cy, cz = getWorldTranslation(cam);
			local x,y,z = getSplinePosition(self.spline, self.splinePosition);
			local distance = MathUtil.vector3Length(x-cx, y-cy, z-cz);
			if distance > self.clipDistance then

				if getVisibility(self.graphId) then
					setVisibility(self.graphId, false);
					if isAnimTrackEnabled(self.animCharSet, self.activeTrack) then
						disableAnimTrack(self.animCharSet, self.activeTrack);
					end;
					if isAnimTrackEnabled(self.animCharSet, self.nextTrack) then
						disableAnimTrack(self.animCharSet, self.nextTrack);
					end;
				end;
			end;
			if distance < self.clipDistance and self.viewable then


				setVisibility(self.graphId, true);
				if not isAnimTrackEnabled(self.animCharSet, self.activeTrack) then
					enableAnimTrack(self.animCharSet, self.activeTrack);
				end;
				self.timer1 = self.timer1 - dt;

				if self.timer1 <= 0 then
					self.timer1 = math.random(5000, 15000);
					self.nextTrack = math.random(1, self.numTracks);
					enableAnimTrack(self.animCharSet, self.nextTrack);
				end;
				if self.nextTrack ~= self.activeTrack then
					self.timer2 = math.min(self.timer2 + 0.05, 1);
					setAnimTrackBlendWeight(self.animCharSet, self.nextTrack, self.timer2);
					if isAnimTrackEnabled(self.animCharSet, self.activeTrack) then
						local ATBW = 1 - getAnimTrackBlendWeight(self.animCharSet, self.nextTrack);
						setAnimTrackBlendWeight(self.animCharSet, self.activeTrack, ATBW);
					end;
					if self.timer2 == 1 then
						self.timer2 = 0;
						disableAnimTrack(self.animCharSet, self.activeTrack);
						setAnimTrackBlendWeight(self.animCharSet, self.activeTrack, 1);
						self.activeTrack = self.nextTrack;
					end;
				end;
				if self.walkTracks[self.activeTrack] > 0 or self.walkTracks[self.nextTrack] > 0 then
					local speed = self.walkTracks[self.activeTrack];
					if self.walkTracks[self.nextTrack] > self.walkTracks[self.activeTrack] then
						speed = MathUtil.clamp(self.walkTracks[self.nextTrack]*self.timer2, self.walkTracks[self.activeTrack], self.walkTracks[self.nextTrack]);
					elseif self.walkTracks[self.nextTrack] < self.walkTracks[self.activeTrack] then
						speed = math.max(self.walkTracks[self.activeTrack]*(1 - self.timer2), self.walkTracks[self.nextTrack]);
					end;
					local timeScale = (speed/3.6) / self.splineLength;
					if self.parent then
						self.splinePosition = self.parent.splinePosition;
					else
						self.splinePosition = self.splinePosition - 0.001*dt*timeScale;

					end;
				end;
				local x,y,z = getSplinePosition(self.spline, self.splinePosition);
				local rx,ry,rz = getSplineOrientation(self.spline, self.splinePosition, 0, -1, 0);
				setTranslation(self.animRoot, x, y, z);
				setRotation(self.animRoot, rx,ry,rz);
				if self.sound then
					self.soundTimer = self.soundTimer - dt;
					if self.soundTimer <= 0 then
						if getVisibility(self.sound) then
							setVisibility(self.sound, false);
							self.soundTimer = math.random(15000, 60000);
						else
							local x,y,z = getWorldTranslation(self.animRoot);
							setTranslation(self.sound, x, y, z);
							setVisibility(self.sound, true);
							self.soundTimer = self.soundDuration+500;
						end;
					end;
				end;
			end;
		else
			if getVisibility(self.graphId) then
				setVisibility(self.graphId, false);
			end;
			if isAnimTrackEnabled(self.animCharSet, self.activeTrack) then
					disableAnimTrack(self.animCharSet, self.activeTrack);
			end;
			if isAnimTrackEnabled(self.animCharSet, self.nextTrack) then
				disableAnimTrack(self.animCharSet, self.nextTrack);
			end;
		end;
	else
	print("is NOT enabled")
	end;


	else
		self.initCount = self.initCount + 1
		setVisibility(self.animRoot, true)
	end

end;

function animatedCharacter:animatedCharacterTimerCallback()
    if g_currentMission ~= nil then
   
        if self.startTime > self.endTime then -- goes through midnight
            self.isActive = g_currentMission.environment.dayTime > self.startTime*60*60*1000 or g_currentMission.environment.dayTime < self.endTime*60*60*1000;
        else
            self.isActive = g_currentMission.environment.dayTime > self.startTime*60*60*1000 and g_currentMission.environment.dayTime < self.endTime*60*60*1000;
        end;

 
        self.viewable = self.isActive;
 
 
    end
  
    return true;
end;

g_onCreateUtil.addOnCreateFunction("animatedCharacter", animatedCharacter.onCreate);
 --- Log Info ---
local function author() for i=1,table.getn(metadata) do local _,n=string.find(metadata[i],"## Author: ");if n then return (string.sub (metadata[i], n+1)); end;end;end;
local function name() for i=1,table.getn(metadata) do local _,n=string.find(metadata[i],"## Title: ");if n then return (string.sub (metadata[i], n+1)); end;end;end;
local function version() for i=1,table.getn(metadata) do local _,n=string.find(metadata[i],"## Version: ");if n then return (string.sub (metadata[i], n+1)); end;end;end;
local function datum() for i=1,table.getn(metadata) do local _,n=string.find(metadata[i],"## Date: ");if n then return (string.sub (metadata[i], n+1)); end;end;end;
local function support() for i=1,table.getn(metadata) do local _,n=string.find(metadata[i],"## Web: ");if n then return (string.sub (metadata[i], n+1)); end;end;end;
print("Script "..(name()).." v"..(version()).." "..(datum()).." by "..(author()).." loaded! Support on "..(support()));